@@ -55,7 +55,7 @@
 #'   See details below.
 #' @param poll_connection Whether to have a control connection to
 #'   the process. This is used to transmit messages from the subprocess
-#'   to the parent.
+#'   to the main process.
 #' @param cmdargs Command line arguments to pass to the R process.
 #'   Note that `c("-f", rscript)` is appended to this, `rscript`
 #'   is the name of the script file to run. This contains a call to the
@@ -77,6 +77,10 @@
 #'   `show = TRUE` and the R session is interactive.
 #' @param system_profile Whether to use the system profile file.
 #' @param user_profile Whether to use the user's profile file.
+#'   If this is `"project"`, then only the profile from the working
+#'   directory is used, but the `R_PROFILE_USER` environment variable
+#'   and the user level profile are not. See also "Security considerations"
+#'   below.
 #' @param env Environment variables to set for the child process.
 #' @param timeout Timeout for the function call to finish. It can be a
 #'   [base::difftime] object, or a real number, meaning seconds.
@@ -90,16 +94,17 @@
 #'
 #' `callr` handles errors properly. If the child process throws an
 #' error, then `callr` throws an error with the same error message
-#' in the parent process.
+#' in the main process.
 #'
 #' The `error` expert argument may be used to specify a different
 #' behavior on error. The following values are possible:
-#' * `error` is the default behavior: throw an error in the parent, with
-#'   the same error message. In fact the same error object is thrown again.
-#' * `stack` also throws an error in the parent, but the error
+#' * `error` is the default behavior: throw an error in the main process,
+#'   with a prefix and the same error message as in the subprocess.
+#' * `stack` also throws an error in the main process, but the error
 #'   is of a special kind, class `callr_error`, and it contains
 #'   both the original error object, and the call stack of the child,
-#'   as written out by [utils::dump.frames()].
+#'   as written out by [utils::dump.frames()]. This is now deprecated,
+#'   because the error thrown for `"error"` has the same information.
 #' * `debugger` is similar to `stack`, but in addition
 #'   to returning the complete call stack, it also start up a debugger
 #'   in the child call stack, via [utils::debugger()].
@@ -107,6 +112,19 @@
 #' The default error behavior can be also set using the `callr.error`
 #' option. This is useful to debug code that uses `callr`.
 #'
+#' callr uses parent errors, to keep the stacks of the main process and the
+#' subprocess(es) in the same error object. 
+#'
+#' @section Security considerations:
+#'
+#' `callr` makes a copy of the user's `.Renviron` file and potentially of
+#' the local or user `.Rprofile`, in the session temporary
+#' directory. Avoid storing sensitive information such as passwords, in
+#' your environment file or your profile, otherwise this information will
+#' get scattered in various files, at least temporarily, until the
+#' subprocess finishes. You can use the keyring package to avoid passwords
+#' in plain files.
+#'
 #' @family callr functions
 #' @examples
 #' \dontrun{
@@ -128,7 +146,7 @@ r <- function(func, args = list(), libpath = .libPaths(),
               cmdargs = c("--slave", "--no-save", "--no-restore"),
               show = FALSE, callback = NULL,
               block_callback = NULL, spinner = show && interactive(),
-              system_profile = FALSE, user_profile = FALSE,
+              system_profile = FALSE, user_profile = "project",
               env = rcmd_safe_env(), timeout = Inf, ...) {
 
   ## This contains the context that we set up in steps
