@@ -78,7 +78,8 @@ test_that("parent errors", {
     error = function(e) e)
 
   expect_s3_class(err, "rlib_error")
-  expect_s3_class(err$parent, "simpleError")
+  expect_s3_class(err$parent, "callr_remote_error")
+  expect_s3_class(err$parent$error, "simpleError")
   expect_match(
     conditionMessage(err),
     "callr subprocess failed: non-numeric argument")
@@ -98,7 +99,8 @@ test_that("parent errors, another level", {
 
   expect_s3_class(err, "rlib_error")
   expect_s3_class(err$parent, "rlib_error")
-  expect_s3_class(err$parent$parent, "simpleError")
+  expect_s3_class(err$parent$parent, "callr_remote_error")
+  expect_s3_class(err$parent$parent$error, "simpleError")
 
   expect_match(
     conditionMessage(err),
@@ -140,7 +142,7 @@ test_that("error traces are merged", {
   expect_s3_class(cond$parent, "error")
   expect_s3_class(cond$trace, "rlib_trace")
 
-  expect_equal(length(cond$trace$nframe), 2)
+  expect_equal(length(cond$trace$nframes), 2)
   expect_true(cond$trace$nframe[1] < cond$trace$nframe[2])
   expect_match(cond$trace$messages[[1]], "subprocess failed: non-numeric")
   expect_match(cond$trace$messages[[2]], "non-numeric argument")
@@ -160,7 +162,8 @@ test_that("errors in r_bg() are merged", {
     error = function(e) e)
 
   expect_s3_class(err, "rlib_error")
-  expect_s3_class(err$parent, "simpleError")
+  expect_s3_class(err$parent, "callr_remote_error")
+  expect_s3_class(err$parent$error, "simpleError")
   expect_match(
     conditionMessage(err),
     "callr subprocess failed: non-numeric argument")
@@ -183,7 +186,8 @@ test_that("errors in r_process are merged", {
     error = function(e) e)
 
   expect_s3_class(err, "rlib_error")
-  expect_s3_class(err$parent, "simpleError")
+  expect_s3_class(err$parent, "callr_remote_error")
+  expect_s3_class(err$parent$error, "simpleError")
   expect_match(
     conditionMessage(err),
     "callr subprocess failed: non-numeric argument")
@@ -206,7 +210,8 @@ test_that("errors in r_session$run() are merged", {
 
   for (i in seq_along(err)) {
     expect_s3_class(err[[i]], "rlib_error")
-    expect_s3_class(err[[i]]$parent, "simpleError")
+    expect_s3_class(err[[i]]$parent, "callr_remote_error")
+    expect_s3_class(err[[i]]$parent$error, "simpleError")
     expect_match(
       conditionMessage(err[[i]]),
       "callr subprocess failed: non-numeric argument")
@@ -232,7 +237,8 @@ test_that("errors in r_session$call() are merged", {
 
   for (i in seq_along(err)) {
     expect_s3_class(err[[i]], "rlib_error")
-    expect_s3_class(err[[i]]$parent, "simpleError")
+    expect_s3_class(err[[i]]$parent, "callr_remote_error")
+    expect_s3_class(err[[i]]$parent$error, "simpleError")
     expect_match(
       conditionMessage(err[[i]]),
       "callr subprocess failed: non-numeric argument")
@@ -241,3 +247,12 @@ test_that("errors in r_session$call() are merged", {
       "non-numeric argument")
   }
 })
+
+test_that("child error is not modified", {
+  err <- tryCatch(callr::r(function() stop("foobar")), error = function(e) e)
+  expect_identical(
+    class(err$parent$error),
+    c("simpleError", "error", "condition")
+  )
+  expect_null(err$parent$error$trace)
+})
